@@ -7,9 +7,9 @@ from functools import partial
 from sql import Table, Null
 from sql.functions import Overlay, Position
 from sql.operators import Concat
-from sql.conditionals import Case
 
-from trytond.model import Workflow, ModelView, ModelSQL, fields
+from trytond.model import Workflow, ModelView, ModelSQL, fields, \
+    sequence_ordered
 from trytond.modules.company import CompanyReport
 from trytond.wizard import Wizard, StateAction, StateView, StateTransition, \
     Button
@@ -29,6 +29,14 @@ __all__ = ['Sale', 'SaleIgnoredInvoice', 'SaleRecreatedInvoice',
     'ReturnSaleStart', 'ReturnSale']
 
 _ZERO = Decimal(0)
+STATES = [
+    ('draft', 'Draft'),
+    ('quotation', 'Quotation'),
+    ('confirmed', 'Confirmed'),
+    ('processing', 'Processing'),
+    ('done', 'Done'),
+    ('cancel', 'Canceled'),
+    ]
 
 
 class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
@@ -51,14 +59,7 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             'readonly': Eval('state') != 'draft',
             },
         depends=['state'])
-    state = fields.Selection([
-        ('draft', 'Draft'),
-        ('quotation', 'Quotation'),
-        ('confirmed', 'Confirmed'),
-        ('processing', 'Processing'),
-        ('done', 'Done'),
-        ('cancel', 'Canceled'),
-    ], 'State', readonly=True, required=True)
+    state = fields.Selection(STATES, 'State', readonly=True, required=True)
     sale_date = fields.Date('Sale Date',
         states={
             'readonly': ~Eval('state').in_(['draft', 'quotation']),
@@ -66,7 +67,8 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             },
         depends=['state'])
     payment_term = fields.Many2One('account.invoice.payment_term',
-        'Payment Term', required=True, states={
+        'Payment Term',
+        states={
             'readonly': Eval('state') != 'draft',
             },
         depends=['state'])
@@ -341,6 +343,9 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
         table = TableHandler(cls, module_name)
         table.index_action('create_date', action='add')
 
+        # Migration from 4.0: Drop not null on payment_term
+        table.not_null_action('payment_term', 'remove')
+
     @classmethod
     def default_payment_term(cls):
         PaymentTerm = Pool().get('account.invoice.payment_term')
@@ -645,8 +650,11 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             self.raise_user_error('invalid_method', (self.rec_name,))
 
     def get_rec_name(self, name):
-        return (self.number or str(self.id)
-            + ' - ' + self.party.rec_name)
+        if self.number:
+            return self.number
+        elif self.reference:
+            return '[%s]' % self.reference
+        return '(%s)' % self.id
 
     @classmethod
     def search_rec_name(cls, name, clause):
@@ -655,13 +663,10 @@ class Sale(Workflow, ModelSQL, ModelView, TaxableMixin):
             bool_op = 'AND'
         else:
             bool_op = 'OR'
-        names = value.split(' - ', 1)
         domain = [bool_op,
-            ('number', operator, names[0]),
-            ('reference', operator, names[0]),
+            ('number', operator, value),
+            ('reference', operator, value),
             ]
-        if len(names) != 1 and names[1]:
-            domain = [bool_op, domain, ('party', operator, names[1])]
         return domain
 
     @classmethod
@@ -947,53 +952,61 @@ class SaleRecreatedInvoice(ModelSQL):
             ondelete='RESTRICT', select=True, required=True)
 
 
-class SaleLine(ModelSQL, ModelView):
+class SaleLine(sequence_ordered(), ModelSQL, ModelView):
     'Sale Line'
     __name__ = 'sale.line'
     _rec_name = 'description'
     sale = fields.Many2One('sale.sale', 'Sale', ondelete='CASCADE',
-        select=True)
-    sequence = fields.Integer('Sequence')
+        select=True,
+        states={
+            'readonly': ((Eval('sale_state') != 'draft')
+                & Bool(Eval('sale'))),
+            },
+        depends=['sale_state'])
     type = fields.Selection([
         ('line', 'Line'),
         ('subtotal', 'Subtotal'),
         ('title', 'Title'),
         ('comment', 'Comment'),
-        ], 'Type', select=True, required=True)
+        ], 'Type', select=True, required=True,
+        states={
+            'readonly': Eval('sale_state') != 'draft',
+            },
+        depends=['sale_state'])
     quantity = fields.Float('Quantity',
         digits=(16, Eval('unit_digits', 2)),
         states={
             'invisible': Eval('type') != 'line',
             'required': Eval('type') == 'line',
-            'readonly': ~Eval('_parent_sale', {}),
+            'readonly': Eval('sale_state') != 'draft',
             },
-        depends=['type', 'unit_digits'])
+        depends=['type', 'unit_digits', 'sale_state'])
     unit = fields.Many2One('product.uom', 'Unit', ondelete='RESTRICT',
             states={
                 'required': Bool(Eval('product')),
                 'invisible': Eval('type') != 'line',
-                'readonly': ~Eval('_parent_sale', {}),
+                'readonly': Eval('sale_state') != 'draft',
             },
         domain=[
             If(Bool(Eval('product_uom_category')),
                 ('category', '=', Eval('product_uom_category')),
                 ('category', '!=', -1)),
             ],
-        depends=['product', 'type', 'product_uom_category'])
+        depends=['product', 'type', 'product_uom_category', 'sale_state'])
     unit_digits = fields.Function(fields.Integer('Unit Digits'),
         'on_change_with_unit_digits')
     product = fields.Many2One('product.product', 'Product',
         ondelete='RESTRICT', domain=[('salable', '=', True)],
         states={
             'invisible': Eval('type') != 'line',
-            'readonly': ~Eval('_parent_sale', {}),
+            'readonly': Eval('sale_state') != 'draft',
             },
         context={
             'locations': If(Bool(Eval('_parent_sale', {}).get('warehouse')),
                 [Eval('_parent_sale', {}).get('warehouse', 0)], []),
             'stock_date_end': Eval('_parent_sale', {}).get('sale_date'),
             'stock_skip_warehouse': True,
-            }, depends=['type'])
+            }, depends=['type', 'sale_state'])
     product_uom_category = fields.Function(
         fields.Many2One('product.uom.category', 'Product Uom Category'),
         'on_change_with_product_uom_category')
@@ -1001,15 +1014,19 @@ class SaleLine(ModelSQL, ModelView):
         states={
             'invisible': Eval('type') != 'line',
             'required': Eval('type') == 'line',
-            }, depends=['type'])
+            'readonly': Eval('sale_state') != 'draft'
+            }, depends=['type', 'sale_state'])
     amount = fields.Function(fields.Numeric('Amount',
             digits=(16, Eval('_parent_sale', {}).get('currency_digits', 2)),
             states={
                 'invisible': ~Eval('type').in_(['line', 'subtotal']),
-                'readonly': ~Eval('_parent_sale'),
                 },
-            depends=['type']), 'get_amount')
-    description = fields.Text('Description', size=None, required=True)
+            depends=['type', 'sale_state']), 'get_amount')
+    description = fields.Text('Description', size=None, required=True,
+        states={
+            'readonly': Eval('sale_state') != 'draft',
+            },
+        depends=['sale_state'])
     note = fields.Text('Note')
     taxes = fields.Many2Many('sale.line-account.tax', 'line', 'tax', 'Taxes',
         order=[('tax.sequence', 'ASC'), ('tax.id', 'ASC')],
@@ -1021,7 +1038,8 @@ class SaleLine(ModelSQL, ModelView):
             ],
         states={
             'invisible': Eval('type') != 'line',
-            }, depends=['type'])
+            'readonly': Eval('sale_state') != 'draft',
+            }, depends=['type', 'sale_state'])
     invoice_lines = fields.One2Many('account.invoice.line', 'origin',
         'Invoice Lines', readonly=True)
     moves = fields.One2Many('stock.move', 'origin', 'Moves', readonly=True)
@@ -1044,11 +1062,12 @@ class SaleLine(ModelSQL, ModelView):
                 },
             depends=['type']),
         'on_change_with_shipping_date')
+    sale_state = fields.Function(fields.Selection(STATES, 'Sale State'),
+        'on_change_with_sale_state')
 
     @classmethod
     def __setup__(cls):
         super(SaleLine, cls).__setup__()
-        cls._order.insert(0, ('sequence', 'ASC'))
         cls._error_messages.update({
                 'customer_location_required': (
                     'Sale "%(sale)s" is missing the '
@@ -1077,11 +1096,6 @@ class SaleLine(ModelSQL, ModelView):
         # Migration from 2.4: drop required on sequence
         table.not_null_action('sequence', action='remove')
 
-    @staticmethod
-    def order_sequence(tables):
-        table, _ = tables[None]
-        return [Case((table.sequence == Null, 0), else_=1), table.sequence]
-
     @staticmethod
     def default_type():
         return 'line'
@@ -1181,7 +1195,7 @@ class SaleLine(ModelSQL, ModelView):
         self.taxes = taxes
 
         category = self.product.sale_uom.category
-        if not self.unit or self.unit not in category.uoms:
+        if not self.unit or self.unit.category != category:
             self.unit = self.product.sale_uom
             self.unit_digits = self.product.sale_uom.digits
 
@@ -1287,6 +1301,11 @@ class SaleLine(ModelSQL, ModelView):
             date = self.sale.sale_date if self.sale else None
             return self.product.compute_shipping_date(date=date)
 
+    @fields.depends('sale', '_parent_sale.state')
+    def on_change_with_sale_state(self, name=None):
+        if self.sale:
+            return self.sale.state
+
     def get_invoice_line(self):
         'Return a list of invoice lines for sale line'
         pool = Pool()
@@ -1527,6 +1546,11 @@ class SaleLineRecreatedMove(ModelSQL):
 class SaleReport(CompanyReport):
     __name__ = 'sale.sale'
 
+    @classmethod
+    def execute(cls, ids, data):
+        with Transaction().set_context(address_with_party=True):
+            return super(SaleReport, cls).execute(ids, data)
+
 
 class OpenCustomer(Wizard):
     'Open Customers'
